<?php

/**
 * @file
 * Theme and template preprocess functions for admin pages.
 */

/**
 * Theme affiliate store category mapping page.
 */
function theme_affiliate_store_category_map_page($form) {
  $has_term = isset($form['term']) && is_array($form['term']);
  $header = array(
    $has_term ? theme('table_select_header_cell') : '',
    array('data' => t('Term'), 'field' => 'td.name', 'sort' => 'asc'),
    array('data' => t('Synonym'), 'field' => 'ts.name'),
    array('data' => t('Operations')),
  );
  $output = '';

  $rows = array();
  if ($has_term) {
    foreach (element_children($form['term']) as $key) {
      $rows[] = array(
        drupal_render($form['checkboxes'][$key]),
        drupal_render($form['term'][$key]),
        drupal_render($form['synonym'][$key]),
        drupal_render($form['operations'][$key]['edit']),
      );
    }
  }
  else {
    $rows[] = array(
      array(
        'data' => '<em>' . t('No categories imported yet.') . '</em>',
        'colspan' => 4,
      ),
    );
  }

  $output .= theme('table', $header, $rows);
  $output .= drupal_render($form['pager']);
  $output .= drupal_render($form);
  return $output;
}

/**
 * Theme affiliate store subscriptions page.
 *
 * @ingroup themable
 */
function theme_affiliate_store_subscription_page($form) {
  $has_subscription = isset($form['network']) && is_array($form['network']);
  $header = array(
    $has_subscription ? theme('table_select_header_cell') : '',
    array('data' => t('Network'), 'field' => 'network', 'sort' => 'asc'),
    array('data' => t('Merchant'), 'field' => 'merchant'),
    array('data' => t('Category'), 'field' => 'category'),
    array('data' => t('Subcategory'), 'field' => 'subcategory'),
    array('data' => t('Subsubcategory'), 'field' => 'subsubcategory'),
    array('data' => t('Product'), 'field' => 'product'),
  );
  $output = '';

  $rows = array();
  if ($has_subscription) {
    foreach (element_children($form['network']) as $key) {
      $rows[] = array(
        drupal_render($form['checkboxes'][$key]),
        drupal_render($form['network'][$key]),
        drupal_render($form['merchant'][$key]),
        drupal_render($form['category'][$key]),
        drupal_render($form['subcategory'][$key]),
        drupal_render($form['subsubcategory'][$key]),
        drupal_render($form['product'][$key]),
      );
    }
  }
  else {
    $rows[] = array(
      array(
        'data' => '<em>' . t('No subscriptions imported yet.') . '</em>',
        'colspan' => 7,
      ),
    );
  }

  $output .= theme('table', $header, $rows);
  $output .= drupal_render($form['pager']);
  $output .= drupal_render($form);
  return $output;
}

/**
 * Implements theme passthrough hook.
 */
function _affiliate_store_admin_theme(&$theme) {
  $theme['affiliate_store_category_map_page'] = array(
    'arguments' => array('form' => NULL),
    'file' => 'includes/admin.theme.inc',
  );
  $theme['affiliate_store_subscription_page'] = array(
    'arguments' => array('form' => NULL),
    'file' => 'includes/admin.theme.inc',
  );
}
