<?php

/**
 * @file
 * Common and helper functions.
 */

// NOTE: Backport from Drupal 7 PREG_CLASS_UNICODE_WORD_BOUNDARY.
/**
 * Matches Unicode characters that are word boundaries.
 *
 * @see http://unicode.org/glossary
 *
 * Characters with the following General_category (gc) property values are used
 * as word boundaries. While this does not fully conform to the Word Boundaries
 * algorithm described in http://unicode.org/reports/tr29, as PCRE does not
 * contain the Word_Break property table, this simpler algorithm has to do.
 * - Cc, Cf, Cn, Co, Cs: Other.
 * - Pc, Pd, Pe, Pf, Pi, Po, Ps: Punctuation.
 * - Sc, Sk, Sm, So: Symbols.
 * - Zl, Zp, Zs: Separators.
 *
 * Non-boundary characters include the following General_category (gc) property
 * values:
 * - Ll, Lm, Lo, Lt, Lu: Letters.
 * - Mc, Me, Mn: Combining Marks.
 * - Nd, Nl, No: Numbers.
 *
 * Note that the PCRE property matcher is not used because we wanted to be
 * compatible with Unicode 5.2.0 regardless of the PCRE version used (and any
 * bugs in PCRE property tables).
 */
define(
  'AFFILIATE_STORE_UNICODE_WORD_BOUNDARY',
  '\x{0}-\x{2F}\x{3A}-\x{40}\x{5B}-\x{60}\x{7B}-\x{A9}\x{AB}-\x{B1}\x{B4}' .
  '\x{B6}-\x{B8}\x{BB}\x{BF}\x{D7}\x{F7}\x{2C2}-\x{2C5}\x{2D2}-\x{2DF}' .
  '\x{2E5}-\x{2EB}\x{2ED}\x{2EF}-\x{2FF}\x{375}\x{37E}-\x{385}\x{387}\x{3F6}' .
  '\x{482}\x{55A}-\x{55F}\x{589}-\x{58A}\x{5BE}\x{5C0}\x{5C3}\x{5C6}' .
  '\x{5F3}-\x{60F}\x{61B}-\x{61F}\x{66A}-\x{66D}\x{6D4}\x{6DD}\x{6E9}' .
  '\x{6FD}-\x{6FE}\x{700}-\x{70F}\x{7F6}-\x{7F9}\x{830}-\x{83E}' .
  '\x{964}-\x{965}\x{970}\x{9F2}-\x{9F3}\x{9FA}-\x{9FB}\x{AF1}\x{B70}' .
  '\x{BF3}-\x{BFA}\x{C7F}\x{CF1}-\x{CF2}\x{D79}\x{DF4}\x{E3F}\x{E4F}' .
  '\x{E5A}-\x{E5B}\x{F01}-\x{F17}\x{F1A}-\x{F1F}\x{F34}\x{F36}\x{F38}' .
  '\x{F3A}-\x{F3D}\x{F85}\x{FBE}-\x{FC5}\x{FC7}-\x{FD8}\x{104A}-\x{104F}' .
  '\x{109E}-\x{109F}\x{10FB}\x{1360}-\x{1368}\x{1390}-\x{1399}\x{1400}' .
  '\x{166D}-\x{166E}\x{1680}\x{169B}-\x{169C}\x{16EB}-\x{16ED}' .
  '\x{1735}-\x{1736}\x{17B4}-\x{17B5}\x{17D4}-\x{17D6}\x{17D8}-\x{17DB}' .
  '\x{1800}-\x{180A}\x{180E}\x{1940}-\x{1945}\x{19DE}-\x{19FF}' .
  '\x{1A1E}-\x{1A1F}\x{1AA0}-\x{1AA6}\x{1AA8}-\x{1AAD}\x{1B5A}-\x{1B6A}' .
  '\x{1B74}-\x{1B7C}\x{1C3B}-\x{1C3F}\x{1C7E}-\x{1C7F}\x{1CD3}\x{1FBD}' .
  '\x{1FBF}-\x{1FC1}\x{1FCD}-\x{1FCF}\x{1FDD}-\x{1FDF}\x{1FED}-\x{1FEF}' .
  '\x{1FFD}-\x{206F}\x{207A}-\x{207E}\x{208A}-\x{208E}\x{20A0}-\x{20B8}' .
  '\x{2100}-\x{2101}\x{2103}-\x{2106}\x{2108}-\x{2109}\x{2114}' .
  '\x{2116}-\x{2118}\x{211E}-\x{2123}\x{2125}\x{2127}\x{2129}\x{212E}' .
  '\x{213A}-\x{213B}\x{2140}-\x{2144}\x{214A}-\x{214D}\x{214F}' .
  '\x{2190}-\x{244A}\x{249C}-\x{24E9}\x{2500}-\x{2775}\x{2794}-\x{2B59}' .
  '\x{2CE5}-\x{2CEA}\x{2CF9}-\x{2CFC}\x{2CFE}-\x{2CFF}\x{2E00}-\x{2E2E}' .
  '\x{2E30}-\x{3004}\x{3008}-\x{3020}\x{3030}\x{3036}-\x{3037}' .
  '\x{303D}-\x{303F}\x{309B}-\x{309C}\x{30A0}\x{30FB}\x{3190}-\x{3191}' .
  '\x{3196}-\x{319F}\x{31C0}-\x{31E3}\x{3200}-\x{321E}\x{322A}-\x{3250}' .
  '\x{3260}-\x{327F}\x{328A}-\x{32B0}\x{32C0}-\x{33FF}\x{4DC0}-\x{4DFF}' .
  '\x{A490}-\x{A4C6}\x{A4FE}-\x{A4FF}\x{A60D}-\x{A60F}\x{A673}\x{A67E}' .
  '\x{A6F2}-\x{A716}\x{A720}-\x{A721}\x{A789}-\x{A78A}\x{A828}-\x{A82B}' .
  '\x{A836}-\x{A839}\x{A874}-\x{A877}\x{A8CE}-\x{A8CF}\x{A8F8}-\x{A8FA}' .
  '\x{A92E}-\x{A92F}\x{A95F}\x{A9C1}-\x{A9CD}\x{A9DE}-\x{A9DF}' .
  '\x{AA5C}-\x{AA5F}\x{AA77}-\x{AA79}\x{AADE}-\x{AADF}\x{ABEB}' .
  '\x{D800}-\x{F8FF}\x{FB29}\x{FD3E}-\x{FD3F}\x{FDFC}-\x{FDFD}' .
  '\x{FE10}-\x{FE19}\x{FE30}-\x{FE6B}\x{FEFF}-\x{FF0F}\x{FF1A}-\x{FF20}' .
  '\x{FF3B}-\x{FF40}\x{FF5B}-\x{FF65}\x{FFE0}-\x{FFFD}'
);

/**
 * Clear all affiliate store caches.
 */
function affiliate_store_clear_cache() {
  cache_clear_all('affiliate_store', 'cache_block', TRUE);
  $url = url('affiliate-store', array('absolute' => TRUE));
  cache_clear_all($url, 'cache_page', TRUE);
}

/**
 * Convert a text using a specific string case.
 *
 * @param string $text
 *   The text that will be converted.
 * @param string $case
 *   (optional) One of the following string conversion case:
 *   - none
 *   - lower
 *   - upper
 *   - title
 *   - smart
 *   Default to 'smart'.
 *
 * @return string
 *   The converted text.
 */
function affiliate_store_convert_case($text, $case = 'smart') {
  $map = array(
    'none' => '',
    'lower' => 'drupal_strtolower',
    'upper' => 'drupal_strtoupper',
    'title' => 'affiliate_store_ucwords',
    'smart' => 'affiliate_store_smartcase',
  );
  return !empty($map[$case]) ? $map[$case]($text) : $text;
}

/**
 * Helper function for converting term name string case.
 *
 * @param string $text
 *   The term name that will be converted.
 * @param string $case
 *   (optional) Conversion case to use. Default to NULL meaning use conversion
 *   case configured at settings page.
 *
 * @return string
 *   The converted term name.
 */
function affiliate_store_convert_term_case($text, $case = NULL) {
  if ($case === NULL) {
    $case = variable_get('affiliate_store_term_case', 'smart');
  }
  return affiliate_store_convert_case($text, $case);
}

/**
 * Cron job; Update affiliate store.
 */
function affiliate_store_cron_update() {
  // Run as user 1 since the cron must has permission to create, update, or
  // delete nodes.
  affiliate_store_impersonate_user(1);

  affiliate_store_include('batch');
  $max_time = ini_get('max_execution_time');
  // If there is no execution time limit, use 5 minutes.
  $max_time = $max_time ? ($max_time * 0.5) : (300 * 0.5);
  do {
    $context = array(
      'operation' => isset($context['operation']) ? $context['operation'] : 0,
      'max_operation' => variable_get('affiliate_store_update_throttle', 50),
    );
    $result = affiliate_store_batch_dequeue('%', $context, 0.5, 'page');
    if ($result === TRUE) {
      // Log to watchdog if there is result on executed batch.
      $errors = affiliate_store_batch_results($context['results'], 'errors');
      if ($errors) {
        watchdog(
          'affiliate store',
          'Error occurred when updating affiliate store. !result',
          array('!result' => theme('item_list', $errors)),
          WATCHDOG_ERROR
        );
      }
      $messages = affiliate_store_batch_results($context['results']);
      if ($messages) {
        watchdog(
          'affiliate store',
          'Affiliate store updated. !result',
          array('!result' => theme('item_list', $messages))
        );
      }

      if ($context['operation'] >= $context['max_operation']) {
        break;
      }
    }
    // If batch queue emptied, queue another round of update batches.
    elseif ($result === NULL) {
      // Skip if scheduled time not reach yet. Update will be twice a day.
      // Skip if there is still inactive batch waiting to run.
      if ((time() < (variable_get('affiliate_store_last_update', 0) + 43200))
        || affiliate_store_batch_count('%', TRUE)
      ) {
        break;
      }

      affiliate_store_include('subscription');
      affiliate_store_subscription_sync($errors);
      if (!$errors) {
        variable_set('affiliate_store_last_update', time());
      }
    }
    else {
      // Dequeue error, most probably manual update is running. Retry later.
      watchdog(
        'affiliate store',
        'Cannot update affiliate store, probably because manual update is ' .
        'also running.',
        NULL, WATCHDOG_WARNING
      );
      break;
    }
  } while ((timer_read('page') / 1000) < $max_time);

  affiliate_store_revert_user();
}

/**
 * Decode a gzip compressed string.
 *
 * @param string $data
 *   String to decode.
 *
 * @return string
 *   Decoded string. Return NULL if decoding failed.
 */
function affiliate_store_gzdecode($data) {
  $file = tempnam(realpath(file_directory_temp()), 'file');
  $fp = fopen($file, 'wb');
  if (!$fp) {
    return NULL;
  }

  fwrite($fp, $data);
  fclose($fp);
  ob_start();
  readgzfile($file);
  $data = ob_get_clean();
  unlink($file);
  return $data;
}

/**
 * Generate a keyed hash value using the HMAC method.
 *
 * @param string $algo
 *   Name of hashing algorithm.
 * @param string $data
 *   Message to be hashed.
 * @param string $key
 *   Shared secret key used for generating the HMAC variant of the message
 *   digest.
 * @param boolean $raw_output
 *   (optional) When set to TRUE, outputs raw binary data. FALSE outputs
 *   lowercase hexits. Default to FALSE.
 *
 * @return mixed
 *   A string containing the calculated message digest as lowercase hexits
 *   unless raw_output is set to true in which case the raw binary
 *   representation of the message digest is returned. Return NULL if hashing
 *   failed.
 */
function affiliate_store_hash_hmac($algo, $data, $key, $raw_output = FALSE) {
  // PHP >= 5.1.2.
  if (function_exists('hash_hmac')) {
    return hash_hmac($algo, $data, $key, $raw_output);
  }
  // Fall back to using phpseclib library.
  if (module_exists('libraries')) {
    if (include_once libraries_get_path('phpseclib') . '/Crypt/Hash.php') {
      $hash = new Crypt_Hash($algo);
      $hash->setKey($key);
      $output = $hash->hash($data);
      return $raw_output ? $output : bin2hex($output);
    }
  }
  return NULL;
}

/**
 * Decode JSON encoded string into PHP associative array.
 *
 * @param string $json
 *   JSON encoded string.
 * @param boolean &$error
 *   Return TRUE if decoding failed, FALSE otherwise.
 *
 * @return mixed
 *   PHP variable. Return NULL if decoding failed, but check $error to be sure
 *   if error really occurred as JSON can also returns NULL.
 */
function affiliate_store_json_decode($json, &$error) {
  $error = FALSE;
  // PHP >= 5.2.0.
  if (function_exists('json_decode')) {
    return json_decode($json, TRUE);
  }
  // Fall back to using Services_JSON PEAR library.
  if (module_exists('libraries')) {
    if (include_once libraries_get_path('Services_JSON') . '/JSON.php') {
      $services_json = new Services_JSON(SERVICES_JSON_LOOSE_TYPE);
      return $services_json->decode($json);
    }
  }
  $error = TRUE;
  return NULL;
}

/**
 * Insert or update product node.
 *
 * Product node attributes are assembled from three sources that are ordered in
 * the following precedence order: Hub > existing node > default values.
 *
 * The product attributes are first set using that of Hub, then any missing
 * attribute is filled in by existing node attributes in case of update, and
 * finally by default values. Default values are those defined by module and
 * content type workflow settings.
 *
 * @param array &$product
 *   An associative array representing product with the following keys:
 *   - network: Network name.
 *   - merchant: Merchant name.
 *   - category: (optional) Product category name.
 *   - subcategory: (optional) Product subcategory name.
 *   - subsubcategory: (optional) Product subsubcategory name.
 *   - nwtid: (optional) Network term ID. If specified, network name is not
 *     used for term ID look up.
 *   - mtid: (optional) Merchant term ID. If specified, merchant name is not
 *     used for term ID look up.
 *   - ctid: (optional) Product category term ID. If specified, product
 *     category name is not used for term ID look up.
 *   - sctid: (optional) Product subcategory term ID. If specified, product
 *     subcategory name is not used for term ID look up.
 *   - ssctid: (optional) Product subsubcategory term ID. If specified, product
 *     subsubcategory name is not used for term ID look up.
 *   - pid: Product ID.
 *   - title: Product title.
 *   - price: (optional) Product price.
 *   - currency: (optional) Price currency.
 *   - url: Product URL.
 *   - thumb_url: (optional) Product thumbnail URL.
 *   - image_url: (optional) Product image URL.
 *   - description: (optional) Product short description.
 *   - status: Product status integer.
 *   Return the saved product with additional keys from complete node object.
 *
 * @return mixed
 *   One of SAVED_NEW, SAVED_UPDATED, or SAVED_DELETED. NULL if no operation.
 */
function affiliate_store_save_product(&$product) {
  $op = SAVED_NEW;
  // Populate product term IDs. Missing vocabulary term names will be replaced
  // using values specified in settings page. Terms that are not yet exist will
  // be created.
  $product['taxonomy'] = array();
  $where = array();
  $args = array();
  foreach (affiliate_store_vocabularies() as $vocab => $info) {
    $field = $info['as'] . 'tid';
    if (empty($product[$field])) {
      if (empty($product[$vocab])) {
        // Get name of vocabulary missing term.
        $product[$vocab] = affiliate_store_get_term_name(0, $vocab);
      }
      $product[$field] = affiliate_store_get_term_id($product[$vocab], $vocab);
    }
    // Set product taxonomy terms.
    $product['taxonomy'][$product[$field]] = $product[$field];
    $where[] = "$field = %d";
    $args[] = $product[$field];
  }
  $where[] = "pid = '%s'";
  $args[] = $product['pid'];

  // Check if the product already exists in database.
  $nid = db_result(
    db_query(
      "SELECT DISTINCT nid FROM {affiliate_store_product}
      WHERE " . implode(' AND ', $where),
      $args
    )
  );
  if ($nid) {
    // If the product already exists and its status is to be unpublished, just
    // delete the node.
    if (!$product['status']) {
      node_delete($nid);
      return SAVED_DELETED;
    }

    // Reuse existing node attributes since they might have been customized by
    // user.
    $node = node_load($nid);
    $product += (array) $node;
    $product['taxonomy'] += $node->taxonomy;
    $op = SAVED_UPDATED;
  }
  elseif (!$product['status']) {
    // Ignore if the product doesn't exists and its status is to be unpublished.
    return NULL;
  }

  // Set product default attributes values.
  $product += array(
    'uid' => 1,
    'type' => 'affiliate_store_product',
    'description' => '',
    // Filtered HTML.
    'format' => 1,
    'log' => t('Imported from Hub.'),
  );
  // Set product attributes using content type default workflow settings.
  $workflow = variable_get("node_options_$product[type]", array('status'));
  $workflow = array_flip($workflow);
  foreach (array_keys($workflow) as $option) {
    $workflow[$option] = 1;
  }
  $product += $workflow;
  $product['body'] = $product['description'];
  $product['teaser'] = node_teaser($product['body']);

  $node = (object) $product;
  node_save($node);
  $product = (array) $node;
  return $op;
}

/**
 * Save a module managed term.
 *
 * @param string $name
 *   Term permanent name.
 * @param string $vocabulary
 *   Vocabulary logical name of term.
 *
 * @return int
 *   Term ID if successful, 0 otherwise.
 */
function affiliate_store_save_term($name, $vocabulary) {
  $vocabs = affiliate_store_vocabularies();
  if (!isset($vocabs[$vocabulary])) {
    return 0;
  }

  $term = array(
    'vid' => affiliate_store_get_vocabulary_id($vocabulary),
    'name' => (
      // Network or merchant name is saved as is.
      in_array($vocabulary, array('network', 'merchant'), TRUE) ?
      $name : affiliate_store_convert_term_case($name)
    ),
  );
  taxonomy_save_term($term);
  $tid = (int) $term['tid'];

  // If the term is a missing term default value, then assign a synonym to it
  // with a nicer name without the brackets.
  if ($name === affiliate_store_get_term_name(0, $vocabulary)) {
    $synonym = preg_replace('/^<(.*)>$/', '$1', $name);
    $synonym = affiliate_store_convert_term_case($synonym);
    db_query(
      "INSERT INTO {term_synonym} SET tid = %d, name = '%s'", $tid, $synonym
    );
  }

  // Store term permanent name in lowercase.
  db_query(
    "INSERT INTO {affiliate_store_$vocabulary}
    SET {$vocabs[$vocabulary]['as']}tid = %d, title = '%s'",
    $tid, drupal_strtolower($name)
  );
  return $tid;
}

/**
 * Save or update a module managed vocabulary.
 *
 * Existing vocabulary will be updated instead if there is one matching the
 * default display name defined by module.
 *
 * @param string $name
 *   Vocabulary logical name.
 * @param int $vid
 *   (optional) Specify a valid vocabulary ID to update instead of searching
 *   for one matching the default display name. Default to 0 which create
 *   a new vocabulary if no existing one is found.
 *
 * @return int
 *   Vocabulary ID if successful, 0 otherwise.
 */
function affiliate_store_save_vocabulary($name, $vid = 0) {
  // Required settings.
  $vocabs = array(
    'network' => array('required' => 1),
    'merchant' => array('required' => 1),
    'category' => array('required' => 0),
    'subcategory' => array('required' => 0),
    'subsubcategory' => array('required' => 0),
  );
  $vocab_base = array(
    'relations' => 0,
    'hierarchy' => 0,
    'multiple' => 0,
    'tags' => 0,
    'module' => 'affiliate_store',
    'nodes' => array('affiliate_store_product' => 1),
  );
  // Overridable settings.
  $vocabs_extra = array(
    'network' => array(
      'name' => t('Affiliate store networks'),
      'description' => t('Names of affiliate store networks.'),
      'weight' => -10,
    ),
    'merchant' => array(
      'name' => t('Affiliate store merchants'),
      'description' => t('Names of affiliate store merchants.'),
      'weight' => -9,
    ),
    'category' => array(
      'name' => t('Affiliate store categories'),
      'description' => t('Names of affiliate store product categories.'),
      'weight' => -8,
    ),
    'subcategory' => array(
      'name' => t('Affiliate store subcategories'),
      'description' => t('Names of affiliate store product subcategories.'),
      'weight' => -7,
    ),
    'subsubcategory' => array(
      'name' => t('Affiliate store subsubcategories'),
      'description' => t('Names of affiliate store product subsubcategories.'),
      'weight' => -6,
    ),
  );
  if (!isset($vocabs[$name])) {
    return 0;
  }

  $vocab_ori = $vid ? taxonomy_vocabulary_load($vid) : NULL;
  if (!$vocab_ori) {
    // Check if there is existing vocabulary with the same display name.
    $result = db_query(
      "SELECT vid FROM {vocabulary} WHERE name = '%s' AND module = '%s'",
      $vocabs_extra[$name]['name'], 'affiliate_store'
    );
    $vid = (int) db_result($result);
    $vocab_ori = $vid ? taxonomy_vocabulary_load($vid) : NULL;
  }

  $vocab = $vocabs[$name] + $vocab_base;
  if ($vocab_ori) {
    $vocab += (array) $vocab_ori;
    $vocab['nodes'] += $vocab_ori->nodes;
  }
  $vocab += $vocabs_extra[$name];
  taxonomy_save_vocabulary($vocab);
  $vid = (int) $vocab['vid'];
  variable_set("affiliate_store_vid_$name", $vid);
  return $vid;
}

/**
 * Check for affiliate store setup error.
 *
 * @return array
 *   An associative array of error messages keyed by setting names that have
 *   problem. If the array is empty, it means the setup is OK.
 */
function affiliate_store_setup_error() {
  $messages = array();
  if (!variable_get('affiliate_store_api_key', '')) {
    $messages['affiliate_store_api_key'] = t(
      'You have not enter your service API key.'
    );
  }
  if (affiliate_store_json_decode('{"test": ["test"]}', $error) === NULL) {
    $messages['affiliate_store_json_decode'] = t(
      'No support for JSON decoding found. See module README.txt for ' .
      'installation instruction.'
    );
  }
  if (affiliate_store_hash_hmac('sha256', 'test', 'test') === NULL) {
    $messages['affiliate_store_hash_hmac'] = t(
      'No support for HMAC generation found. See module README.txt for ' .
      'installation instruction.'
    );
  }
  return $messages;
}

/**
 * Titlecase words in a string with stop words exception.
 *
 * @param string $text
 *   The text that will be converted.
 * @param mixed $stopwords
 *   (optional) An array of stop words or a string of stop words separated by
 *   comma to be remained in lowercase. Default to NULL meaning use stop words
 *   as configured at settings page.
 *
 * @return string
 *   Input $text in titlecase with stop words in lowercase.
 */
function affiliate_store_smartcase($text, $stopwords = NULL) {
  if ($stopwords === NULL) {
    $stopwords = variable_get(
      'affiliate_store_smartcase_stop_words',
      'a,an,and,as,at,by,for,is,in,into,of,off,on,onto,or,per,the,to,with'
    );
  }

  $text = affiliate_store_ucwords($text);
  if ($stopwords) {
    $stopwords = is_string($stopwords) ? explode(',', $stopwords) : $stopwords;
    $stopwords = implode('|', array_map('drupal_ucfirst', $stopwords));
    $text = preg_replace_callback(
      "/(?<=(?<!:|'|ʼ|'s|ʼs)[" . AFFILIATE_STORE_UNICODE_WORD_BOUNDARY . "])" .
      "(" . preg_quote($stopwords, '/') . ")" .
      "(?=$|[" . AFFILIATE_STORE_UNICODE_WORD_BOUNDARY . "])/u",
      '_affiliate_store_smartcase_callback',
      $text
    );
  }
  return $text;
}

/**
 * Capitalizes the first letter of each word in a UTF-8 string.
 *
 * @param string $text
 *   The text that will be converted.
 *
 * @return string
 *   The input $text with each word capitalized.
 */
function affiliate_store_ucwords($text) {
  return preg_replace_callback(
    '/(^|[' . AFFILIATE_STORE_UNICODE_WORD_BOUNDARY . '])' .
    '([^' . AFFILIATE_STORE_UNICODE_WORD_BOUNDARY . '])/u',
    '_affiliate_store_ucwords_callback',
    $text
  );
}

/**
 * Impersonate another user.
 *
 * @param mixed $new_user
 *   User ID or user object to impersonate.
 *
 * @return object
 *   Current user.
 *
 * @see affiliate_store_revert_user()
 */
function affiliate_store_impersonate_user($new_user) {
  global $user;
  static $user_original = NULL;
  static $old_state = TRUE;

  if ($new_user) {
    // Prevent session saving.
    $user_original = $user;
    $old_state = session_save_session();
    session_save_session(FALSE);
    $user = is_numeric($new_user) ? user_load($new_user) : $new_user;
  }
  elseif ($user_original) {
    // Restore original user and session saving state.
    $user = $user_original;
    session_save_session($old_state);
  }
  return $user;
}

/**
 * Revert to previous user after impersonating.
 *
 * @return object
 *   Current user.
 *
 * @see affiliate_store_impersonate_user()
 */
function affiliate_store_revert_user() {
  return affiliate_store_impersonate_user(NULL);
}

/**
 * Lowercase stop words in affiliate_store_smartcase().
 *
 * @param array $matches
 *   Matched stop words. Elements 0 and 1 must be the full match and the stop
 *   word.
 *
 * @return string
 *   Stop word in lowercase.
 */
function _affiliate_store_smartcase_callback($matches) {
  return drupal_strtolower($matches[1]);
}

/**
 * Helper callback that is used to capitalize the first character of a word.
 *
 * @param array $matches
 *   Matched characters. Elements 0, 1, and 2 must be the full match, the word
 *   boundary, and the first character of the word.
 *
 * @return string
 *   Matched word boundary and uppercase of the first character.
 */
function _affiliate_store_ucwords_callback($matches) {
  // Make exceptions. Prevent apostrophe from being regarded as boundary by
  // AFFILIATE_STORE_UNICODE_WORD_BOUNDARY, so 's would not becomes 'S.
  if (in_array($matches[1], array("'", "ʼ"), TRUE)) {
    return $matches[1] . $matches[2];
  }
  return $matches[1] . drupal_strtoupper($matches[2]);
}
