<?php

/**
 * @file
 * Theme implementation to display an affiliate store product node.
 *
 * Available variables:
 * - $title: The (sanitized) title of the node.
 * - $content: Node body or teaser depending on $teaser flag.
 * - $picture: The authors picture of the node output from
 *   theme_user_picture().
 * - $date: Formatted creation date (use $created to reformat with
 *   format_date()).
 * - $links: Themed links like "Read more", "Add new comment", etc. output
 *   from theme_links().
 * - $name: Themed username of node author output from theme_username().
 * - $node_url: Direct url of the current node.
 * - $terms: the themed list of taxonomy term links output from theme_links().
 * - $submitted: themed submission information output from
 *   theme_node_submitted().
 *
 * Affiliate store product variables:
 * - $nwtid: Network term ID.
 * - $mtid: Merchant term ID.
 * - $ctid: Product category term ID.
 * - $sctid: Product subcategory term ID.
 * - $ssctid: Product subsubcategory term ID.
 * - $pid: Product ID.
 * - $price: Product price.
 * - $currency: Standard 3-letter currency code.
 * - $currency_symbol: Currency symbol from theme_affiliate_store_currency().
 * - $url: Product URL that leads to its landing page.
 * - $go: Product link from theme_affiliate_store_go() with $call_to_action as
 *   anchor text.
 * - $thumb_url: Product thumbnail URL.
 * - $image_url: Product image URL.
 * - $thumbnail: Product thumbnail from theme_image() with product title in alt
 *   and title attributes.
 * - $image: Product image from theme_image() with product title in alt and
 *   title attributes.
 * - $call_to_action: Call to action text as configured at settings page.
 *
 * Other variables:
 * - $node: Full node object. Contains data that may not be safe.
 * - $type: Node type, i.e. story, page, blog, etc.
 * - $comment_count: Number of comments attached to the node.
 * - $uid: User ID of the node author.
 * - $created: Time the node was published formatted in Unix timestamp.
 * - $zebra: Outputs either "even" or "odd". Useful for zebra striping in
 *   teaser listings.
 * - $id: Position of the node. Increments each time it's output.
 *
 * Node status variables:
 * - $teaser: Flag for the teaser state.
 * - $page: Flag for the full page state.
 * - $promote: Flag for front page promotion state.
 * - $sticky: Flags for sticky post setting.
 * - $status: Flag for published status.
 * - $comment: State of comment settings for the node.
 * - $readmore: Flags true if the teaser content of the node cannot hold the
 *   main body content.
 * - $is_front: Flags true when presented in the front page.
 * - $logged_in: Flags true when the current user is a logged-in member.
 * - $is_admin: Flags true when the current user is an administrator.
 *
 * @see template_preprocess()
 * @see template_preprocess_node()
 * @see affiliate_store_preprocess_node()
 */
?>
<div id="node-<?php print $node->nid; ?>" class="node node-affiliate-store-product<?php if ($sticky) { print ' sticky'; } ?><?php if (!$status) { print ' node-unpublished'; } ?> clear-block">

<?php if (!$page): ?>
  <h2><a href="<?php print $node_url; ?>" title="<?php print $title; ?>"><?php print $title; ?></a></h2>
<?php endif; ?>

  <?php if ((!$page && $thumbnail) || ($page && !$image && $thumbnail)): ?>
    <div class="affiliate-store-thumbnail"><?php print $thumbnail; ?></div>
  <?php elseif (($page && $image) || (!$page && !$thumbnail && $image)): ?>
    <div class="affiliate-store-image"><?php print $image; ?></div>
  <?php endif; ?>

  <div class="content-wrapper">
    <div class="meta">
    <?php if ($terms): ?>
      <div class="terms terms-inline"><?php print $terms; ?></div>
    <?php endif;?>
    </div>

    <div class="content">
      <?php print $content; ?>
    </div>

    <?php if ($price): ?>
      <span class="affiliate-store-currency"><?php print $currency_symbol; ?></span>
      <span class="affiliate-store-price"><?php print $price; ?></span>
    <?php endif; ?>

    <div class="affiliate-store-url"><?php print $go; ?></div>
  </div>
</div>
